﻿using Microsoft.UI.Xaml.Controls;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.System;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;

namespace TestProject
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            #region 윈도우 크기를 설정한다.

            double width  = 800d;
            double height = 600d;

            double dpi = (double)DisplayInformation.GetForCurrentView().LogicalDpi;

            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;

            Size windowSize = new Size(width * 96d / dpi, height * 96d / dpi);

            ApplicationView.PreferredLaunchViewSize = windowSize;

            Window.Current.Activate();

            ApplicationView.GetForCurrentView().TryResizeView(windowSize);

            #endregion
            #region 윈도우 제목을 설정한다.

            ApplicationView.GetForCurrentView().Title = "TabView 엘리먼트 : KeyboardAccelerators 속성을 사용해 키보드 가속기 사용하기";

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 탭뷰 로드시 처리하기 - tabView_Loaded(sender, e)

        /// <summary>
        /// 탭뷰 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void tabView_Loaded(object sender, RoutedEventArgs e)
        {
            TabView tabView = sender as TabView;

            for(int i = 0; i < 3; i++)
            {
                TabViewItem item = GetTabViewItem(i);

                tabView.TabItems.Add(item);
            }
        }

        #endregion
        #region 탭뷰 탭 추가 버튼 클릭시 처리하기 - tabView_AddTabButtonClick(tabView, _)

        /// <summary>
        /// 탭뷰 탭 추가 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="tabView">탭뷰</param>
        /// <param name="_">이벤트 인자</param>
        private void tabView_AddTabButtonClick(TabView tabView, object _)
        {
            int index = tabView.TabItems.Count;

            TabViewItem item = GetTabViewItem(index);

            tabView.TabItems.Add(item);
        }

        #endregion
        #region 탭뷰 탭 닫기 요청시 처리하기 - tabView_TabCloseRequested(tabView, e)

        /// <summary>
        /// 탭뷰 탭 닫기 요청시 처리하기
        /// </summary>
        /// <param name="tabView">탭뷰</param>
        /// <param name="e">이벤트 인자</param>
        private void tabView_TabCloseRequested(TabView tabView, TabViewTabCloseRequestedEventArgs e)
        {
            tabView.TabItems.Remove(e.Tab);
        }

        #endregion

        #region 신규 탭 키보드 가속기 호출시 처리하기 - newTabKeyboardAccelerator_Invoked(sender, e)

        /// <summary>
        /// 신규 탭 키보드 가속기 호출시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void newTabKeyboardAccelerator_Invoked(KeyboardAccelerator sender, KeyboardAcceleratorInvokedEventArgs e)
        {
            TabView tabView = e.Element as TabView;

            TabViewItem item = GetTabViewItem(tabView.TabItems.Count);

            tabView.TabItems.Add(item);

            e.Handled = true;
        }

        #endregion
        #region 선택 탭 닫기 키보드 가속기 호출시 처리하기 - closeSelectedTabKeyboardAccelerator_Invoked(sender, e)

        /// <summary>
        /// 선택 탭 닫기 키보드 가속기 호출시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void closeSelectedTabKeyboardAccelerator_Invoked(KeyboardAccelerator sender, KeyboardAcceleratorInvokedEventArgs e)
        {
            TabView tabView = e.Element as TabView;

            TabViewItem item = tabView.SelectedItem as TabViewItem;

            if(item.IsClosable)
            {
                tabView.TabItems.Remove(item);
            }

            e.Handled = true;
        }

        #endregion
        #region 번호 탭 탐색 키보드 가속기 호출시 처리하기 - navigateToNumberedTabKeyboardAccelerator_Invoked(sender, e)

        /// <summary>
        /// 번호 탭 탐색 키보드 가속기 호출시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void navigateToNumberedTabKeyboardAccelerator_Invoked(KeyboardAccelerator sender, KeyboardAcceleratorInvokedEventArgs e)
        {
            TabView tabView = e.Element as TabView;

            int index = 0;

            switch(sender.Key)
            {
                case VirtualKey.Number1 : index = 0;                          break;
                case VirtualKey.Number2 : index = 1;                          break;
                case VirtualKey.Number3 : index = 2;                          break;
                case VirtualKey.Number4 : index = 3;                          break;
                case VirtualKey.Number5 : index = 4;                          break;
                case VirtualKey.Number6 : index = 5;                          break;
                case VirtualKey.Number7 : index = 6;                          break;
                case VirtualKey.Number8 : index = 7;                          break;
                case VirtualKey.Number9 : index = tabView.TabItems.Count - 1; break;
            }

            if(index < tabView.TabItems.Count)
            {
                tabView.SelectedIndex = index;
            }

            e.Handled = true;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 탭뷰 항목 구하기 - GetTabViewItem(index)

        /// <summary>
        /// 탭뷰 항목 구하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>탭뷰 항목</returns>
        private TabViewItem GetTabViewItem(int index)
        {
            TabViewItem item = new TabViewItem
            {
                Header     = $"문서 {index}",
                IconSource = new Microsoft.UI.Xaml.Controls.SymbolIconSource() { Symbol = Symbol.Document }
            };

            Frame frame = new Frame();

            switch(index % 3)
            {
                case 0 :

                    frame.Navigate(typeof(SamplePage1));

                    break;

                case 1 :

                    frame.Navigate(typeof(SamplePage2));

                    break;

                case 2 :

                    frame.Navigate(typeof(SamplePage3));

                    break;
            }

            item.Content = frame;

            return item;
        }

        #endregion
    }
}